function update_M2d(string,varargin)
%
%UPDATE_M2D
%
%       2D specific update function, updates the control values and 
%       positions based on the input string, and possibly the value 
%       passed in as well. Then it calls update_state to update the 
%       user state (rotatation, colorbar, etc.)
%
%

global pvoxM2d PguiData ParsecData WFNData Ptoolbar Phandles;

if (~pvoxM2d.top_down)
    zoom(Phandles.figure1, 'reset');
end

switch string
    case 'slider_z'
        %update z position
        
        pvoxM2d.z=round( (WFNData.n-1)*( varargin{1} )+1 );
        set(pvoxM2d.slider_z,'Value', (pvoxM2d.z-1)/(WFNData.n-1) );
        set(pvoxM2d.edit_z,'String', mat2str( WFNData.Z(1,1,pvoxM2d.z), 4 ) );
        
        graph_M2d(pvoxM2d.axes);

        
    case 'edit_z'
        %update z position
        
        val = str2num( varargin{1} )

        if (max(size(val))==1) && (val < WFNData.radius) && (val > -1*WFNData.radius) && isreal(val)

            val = round( (val/WFNData.step+WFNData.radius/WFNData.step) );
            
            if ~(pvoxM2d.z == val)
                pvoxM2d.z = val;
                graph_M2d(pvoxM2d.axes);
            end

        end

        set(pvoxM2d.slider_z,'Value', (pvoxM2d.z-1)/(WFNData.n-1) );
        set(pvoxM2d.edit_z,'String', mat2str( WFNData.Z(1,1,pvoxM2d.z), 4 ) );


    case 'slider_level'
        %update which energy level of wavefunction to display
        
        val = round( varargin{1} *(WFNData.num_states-1)+1 );

        if ~(pvoxM2d.level == val)
            pvoxM2d.level=val;
            graph_M2d(pvoxM2d.axes);
        end
        
        set(pvoxM2d.slider_level,'Value', (pvoxM2d.level-1)/(WFNData.num_states-1) );
        set(pvoxM2d.edit_level,'String', mat2str(pvoxM2d.level + WFNData.first_state - 1, 4 ) );

        set(pvoxM2d.edit_energy,'String', mat2str( ParsecData.EigenvalueEV(pvoxM2d.level + WFNData.first_state - 1,ParsecData.num_iterations), 4 ) );
        
        if ( PguiData.wfn_universal == 1 )
            pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius WFNData.wfn_min WFNData.wfn_max WFNData.wfn_min WFNData.wfn_max];
        else
            pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level)))) min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level))))];
        end

    case 'edit_level'
        %update which energy level of wavefunction to display
        
        val = round( str2num( varargin{1} ) )

        if (max(size(val))==1) && isreal(val) && (val ~= pvoxM2d.level) && (val >= 1)

            if (val > WFNData.num_states)
                pvoxM2d.level = WFNData.num_states;
            else
                pvoxM2d.level = val;
            end
            graph_M2d(pvoxM2d.axes);
        end

        set(pvoxM2d.slider_level,'Value', (pvoxM2d.level-1)/(WFNData.num_states-1) );
        set(pvoxM2d.edit_level,'String', mat2str(pvoxM2d.level + WFNData.first_state - 1, 4 ) );

        set(pvoxM2d.edit_energy,'String', mat2str( ParsecData.EigenvalueEV(pvoxM2d.level + WFNData.first_state - 1,ParsecData.num_iterations), 4 ) );
        if ( PguiData.wfn_universal == 1 )
            pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius WFNData.wfn_min WFNData.wfn_max WFNData.wfn_min WFNData.wfn_max];
        else
            pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level)))) min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level))))];
        end

    case 'top_down'
        %update whether to display the top down view or the surface
        
        if varargin{1}
            % If topdown, turn off rotation 
            pvoxM2d.top_down=1;
            set(Ptoolbar.rotate,'Enable','off');
            set(pvoxM2d.toggle_top_down,'Value',get(pvoxM2d.toggle_top_down,'Max') );
            set(pvoxM2d.toggle_surface,'Value',get(pvoxM2d.toggle_top_down,'Min') );
        else
            % If surface view, turn on rotation
            pvoxM2d.top_down=0;
            set(Ptoolbar.rotate,'Enable','on');
            set(pvoxM2d.toggle_top_down,'Value',get(pvoxM2d.toggle_top_down,'Min') );
            set(pvoxM2d.toggle_surface,'Value',get(pvoxM2d.toggle_top_down,'Max') );
        end
        
        graph_M2d(pvoxM2d.axes);
        
        global Phandles;
        zoom(Phandles.figure1, 'reset');
        
        
    case {'pot','rho','wfn','everything'}
        %update all controls for change of graph or mode
        
        set(pvoxM2d.slider_z,'Value', (pvoxM2d.z-1)/(WFNData.n-1) );
        set(pvoxM2d.edit_z,'String', mat2str( WFNData.Z(1,1,pvoxM2d.z), 4 ) );
        
        set(pvoxM2d.slider_z,'SliderStep', [ 1/(WFNData.n-1) 1/(WFNData.n-1) ] );
        
        set(pvoxM2d.edit_energy,'String', mat2str( ParsecData.EigenvalueEV(pvoxM2d.level + WFNData.first_state - 1,ParsecData.num_iterations), 4 ) );

        graph_M2d(pvoxM2d.axes);
        switch PguiData.graph
            case 'wfn'
                set(Ptoolbar.wfn,'State','on');
                set(Ptoolbar.pot,'State','off');
                set(Ptoolbar.rho,'State','off');

                set(pvoxM2d.slider_level,'SliderStep', [ 1/(WFNData.num_states-1) 1/(WFNData.num_states-1) ] );

                set(pvoxM2d.slider_level,'Value', (pvoxM2d.level-1)/(WFNData.num_states-1) );
                set(pvoxM2d.edit_level,'String', mat2str(pvoxM2d.level + WFNData.first_state - 1, 4 ) );
                if (~pvoxM2d.top_down)
                    view(135,35);
                    click_reset;
                end
                zoom(Phandles.figure1,'reset');

            case 'pot'
                set(Ptoolbar.wfn,'State','off');
                set(Ptoolbar.pot,'State','on');
                set(Ptoolbar.rho,'State','off');
                if (~pvoxM2d.top_down)
                    view(135,35);
                    click_reset;
                end
                zoom(Phandles.figure1,'reset');

            case 'rho'
                set(Ptoolbar.wfn,'State','off');
                set(Ptoolbar.pot,'State','off');
                set(Ptoolbar.rho,'State','on');
                if (~pvoxM2d.top_down)
                    view(135,35);
                    click_reset;
                end
                zoom(Phandles.figure1,'reset');
        end

        if pvoxM2d.top_down
            set(Ptoolbar.rotate,'Enable','off');
            set(pvoxM2d.toggle_top_down,'Value',get(pvoxM2d.toggle_top_down,'Max') );
            set(pvoxM2d.toggle_surface,'Value',get(pvoxM2d.toggle_top_down,'Min') );
        else
            set(Ptoolbar.rotate,'Enable','on');
            set(pvoxM2d.toggle_top_down,'Value',get(pvoxM2d.toggle_top_down,'Min') );
            set(pvoxM2d.toggle_surface,'Value',get(pvoxM2d.toggle_top_down,'Max') );
        end

        if strcmp(string,'everything')
            if ( PguiData.wfn_universal == 1 )
                pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius WFNData.wfn_min WFNData.wfn_max WFNData.wfn_min WFNData.wfn_max];
            else
                pvoxM2d.wfn_scale = [-WFNData.radius WFNData.radius -WFNData.radius WFNData.radius min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level)))) min(min(min(WFNData.wfn(:,:,:,pvoxM2d.level)))) max(max(max(WFNData.wfn(:,:,:,pvoxM2d.level))))];
            end
        end


    otherwise
end

%update camera and user state
if (~pvoxM2d.top_down)
    zoom(Phandles.figure1, 'out');
else
    view(0, 90);
end
switch( PguiData.graph )
    case 'wfn'
%        axis([-WFNData.radius WFNData.radius -WFNData.radius
%        WFNData.radius min(min(min(min(WFNData.wfn(:,:,:,:))))) max(max(max(max(WFNData.wfn(:,:,:,:))))) min(min(min(min(WFNData.wfn(:,:,:,:))))) max(max(max(max(WFNData.wfn(:,:,:,:)))))]);
        axis(pvoxM2d.wfn_scale);
        
    case 'pot'
        axis(pvoxM2d.pot_scale);
    case 'rho'
        axis(pvoxM2d.rho_scale);
end
axis square;
zoom(Phandles.figure1, 'reset');

update_state;
