function update_Mdos(string,varargin)
%
%UPDATE_MDOS
%
%       Density of states specific update function, updates the control 
%       values and positions based on the input string, and possibly 
%       the value passed in as well. Then it calls update_state to 
%       update the user state (rotatation, colorbar, etc.)
%
%

global pvoxMdos ParsecData;

switch string
    case 'occupied'
        %update whether to display occupied levels

        if (pvoxMdos.occupied ~= varargin{1})
            pvoxMdos.occupied = varargin{1};
            set(pvoxMdos.toggle_occupied,'Value',pvoxMdos.occupied);
            graph_Mdos(pvoxMdos.axes);
        end

        
    case 'cumulative'
        %update whether to display occupied levels
        
        if (pvoxMdos.cumulative ~= varargin{1})
            pvoxMdos.cumulative = varargin{1};
            set(pvoxMdos.toggle_cumulative,'Value',pvoxMdos.cumulative);
            graph_Mdos(pvoxMdos.axes);
        end
        
        
    case 'slider_iter'
        %update iteration to display 

        val = round( varargin{1} *(ParsecData.num_iterations-1)+1 );

        if ~(pvoxMdos.iteration == val)
            pvoxMdos.iteration=val;
            graph_Mdos(pvoxMdos.axes);
        end

        set(pvoxMdos.slider_iter,'Value', (pvoxMdos.iteration-1)/(ParsecData.num_iterations-1) );
        set(pvoxMdos.edit_iter,'String', mat2str(pvoxMdos.iteration, 4 ) );
  
        
    case 'edit_iter'
        %update iteration to display
        
        val = round( str2num( varargin{1} ) );

        if (max(size(val)) == 1) && (val >= 1) && (~(pvoxMdos.iteration == val)) && isreal(val) 
            if val > ParsecData.num_iterations
                pvoxMdos.iteration = ParsecData.num_iterations;
            else
                pvoxMdos.iteration = val;
            end
            graph_Mdos(pvoxMdos.axes);
        end
        
        set(pvoxMdos.slider_iter,'Value', (pvoxMdos.iteration-1)/(ParsecData.num_iterations-1) );
        set(pvoxMdos.edit_iter,'String', mat2str(pvoxMdos.iteration, 4 ) );
        
        
    case 'edit_width'
        %update width parameter
        
        val = str2num( varargin{1} );
        
        if (max(size(val)) == 1) && (val > 0) && (~(pvoxMdos.width == val)) && isreal(val) && ~isinf(val)
            pvoxMdos.width = val;
            graph_Mdos(pvoxMdos.axes);
        end
        
        set(pvoxMdos.edit_width,'String', mat2str(pvoxMdos.width, 4 ) );
        
        
    case {'pot','rho','wfn','everything'}
        %update all controls after a change in graph or mode
        
        graph_Mdos(pvoxMdos.axes);
        
        set(pvoxMdos.slider_iter,'Value', (pvoxMdos.iteration-1)/(ParsecData.num_iterations-1) );
        set(pvoxMdos.edit_iter,'String', mat2str(pvoxMdos.iteration, 4 ) );

        set(pvoxMdos.edit_width,'String', mat2str(pvoxMdos.width, 4 ) );
        
        set(pvoxMdos.slider_iter,'SliderStep', [ 1/(ParsecData.num_iterations-1) 1/(ParsecData.num_iterations-1) ] );

        set(pvoxMdos.toggle_occupied,'Value',pvoxMdos.occupied);
        set(pvoxMdos.toggle_cumulative,'Value',pvoxMdos.cumulative);

        grid off;  
        click_reset;

        
    otherwise
end

update_state;
