## PARSEC_PARSER.py - Parses parsec.out from PARSEC ##
import re, sys

## OPEN parsec.out file ##
fname = file('./pTemp/fname.dat', 'r')
fileName = fname.readline()
fname.close()
fileName = fileName.rstrip('\n')
parsec = file(fileName, 'r')

## CREATE regular expressions ##
reNumStates= re.compile("^ Number of states")
reAtomData = re.compile("^ Atom data:")
reNumAtoms = re.compile("^ Total number of atoms")
reSetup    = re.compile("^Setup time")
reDiagTime = re.compile("^Diag")
reFermi    = re.compile("^ Fermi")
reNewState = re.compile("^   State")
reHartTime = re.compile("^Hart")
reChargeD  = re.compile("^   max and min")
reChargeD2 = re.compile("^ Max and min")
reEnergy   = re.compile("^   Eigen")
reSRE      = re.compile("SRE of pot")
reSelfCnst = re.compile("^Time for self")
reDipole   = re.compile("^ Dipole components")
#reAtom     = re.compile("^at ")
#reLocal    = re.compile("^ Coordinates and local")
#reNonLocal = re.compile("^ Reporting non-local")
reTotForce = re.compile("^             total \(net\) force")
#reTotForce2= re.compile("^ coordinates and total forces ")
reTotTime  = re.compile("^ Total CPU")
reNum      = re.compile("-?[\d\.]+")
reExpNum   = re.compile("\.\d+E[\-|+][\d]+")

## FUNCTIONS ##
# NumParse - Find a number in a given string and return as a float
def NumParse(inString):
    try:
        temp = reNum.search(inString)
        temp = temp.group()
        return float(temp)
    except:
        print "NumParse: Error at pos("+str(parsec.tell())+"):"
        print inString
        parsec.close()
        sys.exit()
# NumsParse - Find a list of numbers in a given string and return as a list of floats
def NumsParse(inString):
    try:
        temp = re.findall("[^a-zA-z]-?[\d\.]+",inString) 
        for n in range(0, len(temp)): # Convert to floats
            temp[n] = float(temp[n])
        return temp
    except:
        print "NumsParse: Error at pos("+str(parsec.tell())+"):"
        print inString
        parsec.close()
        sys.exit()

## CREATE data structures ##
numStates = 0  # Stores the number of states computed
numAtoms  = 0  # Stores the number of atoms computed
timeSetup = 0  # Stores Setup time
timeDiag  = [] # Stores Diagonalization times [sec]
timeSelfC = 0  # Stores Time to reach self-consistent field [sec]
timeHart  = [] # Stores containing Hartree times [sec]
timeTotal = 0  # Stores Total Cpu Time [sec]
timeWall  = 0  # Stores Wall-clock time [sec]
Eigenvalues = [] # Stores Eigenvalues
Energies    = [] # Stores Energies
SREpot      = [] # Stores SRE of the pot
ChgWghtpot  = [] # Stores Charge weighted pot
FermiLevel  = [] # Stores Fermi levels [Ry]
ChargeD     = [] # Stores max/min charge densities [e/bohr^3]
Dipole      = [] # Stores Dipole (x,y,z)
#forceIon    = [] # Stores (x,y,z) components of ion forces
#forceSelfC  = [] # Stores (x,y,z) components of Self-consitency forces
#forceCoreC  = [] # Stores (x,y,z) components of Core-correction forces
#forceLocal  = [] # Stores (x,y,z) coordinates and components of local forces
#forceNonLcl = [] # Stores (x,y,z) coordinates and components of non-local forces
#forceTorque = [] # Stores (x,y,z) coordinates and components of torque forces
forceTotal  = [] # Stores (x,y,z) coordinates and components of total forces (after settign net force to zero)
netForce    = [] # Stores net force (x,y,z)
#forceTotal2 = [] # Stores (x,y,z) coordinates and components of total forces (after setting net force to zero)
netTorque   = [] # Stores net torque (x,y,z)

## BEGIN Parsing ##
input = parsec.readline()

## SEARCH for 'Number of states' ##
while (reNumStates.match(input) < 0):
    input = parsec.readline()
numStates = int(NumParse(input))

## SEARCH for Atom data ##
while (reAtomData.match(input) < 0):
    input = parsec.readline()

## PARSE Atom data ##
input = parsec.readline()
input = parsec.readline()
if(input == '\n'):
    input = parsec.readline()

totalTypes = NumParse(input)
fAtom = file('./pTemp/atom.dat', 'w')
for n in range(1,int(totalTypes)+1):
    input = parsec.readline()
    if(input == '\n'):
        input = parsec.readline()
        input = parsec.readline()
        input = parsec.readline()

    num = NumParse(input)
    #fAtom.write(input)
    input = input.lstrip(" There are")
    input = input.rstrip("atoms\n")
    fAtom.write(input)
    fAtom.write("\n")
    for i in range(1, int(num)+4):
        input = parsec.readline()
fAtom.close()

## SEARCH for number of atoms ##
while (reNumAtoms.match(input) < 0):
    input = parsec.readline()
numAtoms = int(NumParse(input))

## SEARCH for 'Setup time' ##
while (reSetup.match(input) < 0):
    input = parsec.readline()
timeSetup = NumParse(input)

## PARSE Iteration Data ##
while (reSelfCnst.match(input)<0): # Continue until self-consitency
    input = parsec.readline()
    print input

    ## SEARCH for Diagonalization Time ##
    if (reDiagTime.match(input)):
        timeDiag.append(NumParse(input))

    ## SEARCH for Fermi level ##
    elif (reFermi.match(input)):
        FermiLevel.append(NumParse(input))    

    ## SEARCH for Eigenvalue Table ##
    elif (reNewState.match(input)):
        input = parsec.readline()
        input = parsec.readline()
        while (input != '\n' and input != ' \n'):

            ## READ values into 'Eigenvalues' ##
            Floats = NumsParse(input)                    
            Eigenvalues.append({'State':Floats[0], 'Eigenvalue[Ry]':Floats[1], 'Eigenvalue[eV]':Floats[2], 'Occup':Floats[3], 'Repr':Floats[4]})
            input = parsec.readline()

    ## SEARCH for Hartree Time ##
    elif (reHartTime.match(input)):
        timeHart.append(NumParse(input))

    ## SEARCH for Charge Density ##
    elif (reChargeD.match(input) or reChargeD2.match(input)):
        Floats = re.findall("\.\d+E[\-|+][\d]+", input)
        ChargeD.append({'max':float(Floats[0]), 'min':float(Floats[1])})

    ## SEARCH for Energies Table ##
    elif (reEnergy.match(input)):

        ## READ energy values ##
        nrg = []
        for n in range (0, 7):
	    if (n == 6 and (input == '\n' or input == ' \n')):
                nrg.append(0)
            else:
                nrg.append(NumParse(input))
                input = parsec.readline()

        input = parsec.readline()
        for n in range (0, 2):
            nrg.append(NumParse(input))
            input = parsec.readline()
 
        ## INSERT values into 'Energies ##
        Energies.append({'Eig_Energy':nrg[0], 'H_Energy':nrg[1], 'I_{Vxc*rho}':nrg[2], 'Exc':nrg[3], 'E-I_Energy':nrg[4], 'I-I_Energy':nrg[5],'Enew-Eold':nrg[6], 'Tot_Energy':nrg[7], 'Energy/atom':nrg[8]})

    ## SEARCH for SRE of pot. &  charge weighted pot
    elif (reSRE.search(input)):
        Floats = NumsParse(input)
        SREpot.append(Floats[2])
        ChgWghtpot.append(Floats[3])

    ## CHECK for EOF ##
    elif (input == ''):
        print "EOF reached unexpectedly"
        sys.exit()
## END of Iteration Data ##

## READ Self-consistency time ##
timeSelfC = (NumParse(input))

## PARSE Self-Consitency Data ##
while (input != ''):

    ## SEARCH for Dipole ##
    if(reDipole.match(input)):
        input = parsec.readline()
        Dipole.append(NumParse(input))
        input = parsec.readline()
        Dipole.append(NumParse(input))
        input = parsec.readline()
        Dipole.append(NumParse(input))
  
    ## Search for Total Forces ##
    elif (reTotForce.search(input)):
        netForce = NumsParse(input)
        for n in range(0,6):
            input = parsec.readline()
        while (input != ' \n'):
            Floats = NumsParse(input)
            forceTotal.append({'(x,y,z)':[Floats[1],Floats[2],Floats[3]], '(Fx,Fy,Fz)':[Floats[4],Floats[5],Floats[6]]})
            input = parsec.readline() 
        input = parsec.readline()
        netTorque = NumsParse(input)

    ## SEARCH for Time measurements ##
    elif (reTotTime.match(input)):
        timeTotal = NumParse(input)
        input = parsec.readline()
        timeWall = NumParse(input)
    input = parsec.readline()
## END of Data ##
    
## CLOSE parsec.out ##
parsec.close()

## Write Main Loop info to file ##
fMain = file('./pTemp/main.dat', 'w')
for n in range(0, len(timeDiag)):
    fMain.write("%E %E %E %E %E %E %E %E %E %E %E %E %E %E %E %E\n" % (timeDiag[n], FermiLevel[n], timeHart[n], ChargeD[n]['min'], ChargeD[n]['max'], Energies[n]['Eig_Energy'],Energies[n]['H_Energy'], Energies[n]['I_{Vxc*rho}'], Energies[n]['Exc'], Energies[n]['E-I_Energy'],Energies[n]['I-I_Energy'],Energies[n]['Enew-Eold'],Energies[n]['Tot_Energy'],Energies[n]['Energy/atom'],SREpot[n],ChgWghtpot[n]))
fMain.close()

## Write Eigenvalues to file ##
fEig = file('./pTemp/eigenvalues.dat', 'w')
for n in range(0, len(Eigenvalues)):
    fEig.write("%f %f %f %f %f\n" % (Eigenvalues[n]['State'], Eigenvalues[n]['Eigenvalue[Ry]'], Eigenvalues[n]['Eigenvalue[eV]'], Eigenvalues[n]['Occup'], Eigenvalues[n]['Repr']))
fEig.close()

## ASSERT len(force*) == numAtoms

## Write Forces to file ##
fForces = file('./pTemp/forces.dat', 'w')
for n in range(0, numAtoms):
    fForces.write("NaN NaN NaN ")
fForces.write("\n")
for n in range(0, numAtoms):
    fForces.write("NaN NaN NaN ")
fForces.write("\n")
for n in range(0, numAtoms):
    fForces.write("NaN NaN NaN ")
fForces.close()               

fForces2 = file('./pTemp/forces2.dat', 'w')
for n in range(0, numAtoms):
    fForces2.write("NaN NaN NaN NaN NaN NaN ")
fForces2.write("\n")
for n in range(0, numAtoms):
    fForces2.write("NaN NaN NaN NaN NaN NaN ")
fForces2.write("\n")
for n in range(0, numAtoms):
    fForces2.write("%f %f %f %f %f %f " % (forceTotal[n]['(x,y,z)'][0], forceTotal[n]['(x,y,z)'][1], forceTotal[n]['(x,y,z)'][2], forceTotal[n]['(Fx,Fy,Fz)'][0], forceTotal[n]['(Fx,Fy,Fz)'][1], forceTotal[n]['(Fx,Fy,Fz)'][2]))
fForces2.write("\n")
for n in range(0, numAtoms):
    fForces2.write("%f %f %f %f %f %f " % (forceTotal[n]['(x,y,z)'][0], forceTotal[n]['(x,y,z)'][1], forceTotal[n]['(x,y,z)'][2], forceTotal[n]['(Fx,Fy,Fz)'][0], forceTotal[n]['(Fx,Fy,Fz)'][1], forceTotal[n]['(Fx,Fy,Fz)'][2]))
fForces2.write("\n")
fForces2.close()

fForces3 = file('./pTemp/forces3.dat', 'w')
fForces3.write("%f %f %f\n" % (Dipole[0], Dipole[1], Dipole[2]))
fForces3.write("%f %f %f\n" % (netForce[0], netForce[1], netForce[2]))
fForces3.write("%f %f %f\n" % (netTorque[0], netTorque[1], netTorque[2]))
fForces3.close()

## Write misc to file ##
fOpts = file('./pTemp/misc.dat', 'w')
fOpts.write("%i states computed\n" % numStates)
fOpts.write("%i atoms simulated\n" % numAtoms)
fOpts.write("%f seconds for Setup time\n" % timeSetup)
fOpts.write("%s seconds to reach self-consistent field\n" % timeSelfC)
fOpts.write("%s seconds total cpu time\n" % timeTotal)
fOpts.write("%s seconds of Wall-clock time\n" % timeWall)
fOpts.close()
