 function [data, wfn, D_valid, W_valid] = readOutput_rsdft(outName, datName)
%
%READOUTPUT
%
%       Runs rsdft parser and gathers data
%

D_valid = 0;
W_valid = [0,0,0,0,0];
data = [];
wfn = [];

% Open parsec.out
try
    h = waitbar(0,'Opening rsdft.out');

    fName = fopen('pTemp/fname.dat','w');
    fwrite(fName,outName);
    fclose(fName);
    waitbar(0.1,h);
    !python ./pTools/RSDFT_PARSER.py
    waitbar(0.2,h);
    data.abridged = 0;
    
    inMain = load('pTemp/main.dat');
    data.timeDiag = inMain(:,1);
    data.FermiLevel = inMain(:,2);
    data.timeHart = inMain(:,3);
    data.ChargeDmin = inMain(:,4);
    data.ChargeDmax = inMain(:,5);
    data.Eig_Energy = inMain(:,6);
    data.H_Energy = inMain(:,7);
    data.Int_Vxc_rho = inMain(:,8);
    data.Exc = inMain(:,9);
    data.E_I_Energy = inMain(:,10);
    data.I_I_Energy = inMain(:,11);
    data.Enew_Eold = inMain(:,12);
    data.Tot_Energy = inMain(:,13);
    data.Energy_atom = inMain(:,14);
    data.SREpot = inMain(:,15);
    data.ChrgWghtPot = inMain(:,16);
        
    fOptions = fopen('pTemp/misc.dat');
    opt = fgetl(fOptions);
    data.num_states = str2num(strtok(opt));
    opt = fgetl(fOptions);
    data.num_atoms = str2num(strtok(opt));

    iter = size(data.timeDiag);
    data.num_iterations = iter(1);

    waitbar(0.3,h);

    inEigs = load('pTemp/eigenvalues.dat');
    m = data.num_states;
    n = size(inEigs(:,1));
    n = n(1);
    n = n/m;
    data.State = reshape(inEigs(:,1),m,n);
    data.EigenvalueRY = reshape(inEigs(:,2),m,n);
    data.EigenvalueEV = reshape(inEigs(:,3),m,n);
    data.Occup = reshape(inEigs(:,4),m,n);
    data.Repr = reshape(inEigs(:,5),m,n);

    inForces = load('pTemp/forces.dat');
    forceIon = reshape(inForces(1,:),3,data.num_atoms);
    data.forceIon = transpose(forceIon);
    forceSelfC = reshape(inForces(2,:),3,data.num_atoms);
    data.forceSelfC = transpose(forceSelfC);
    forceCoreC = reshape(inForces(3,:),3,data.num_atoms);
    data.forceCoreC = transpose(forceCoreC);

    inForces2 = load('pTemp/forces2.dat');
    forceLocal = reshape(inForces2(1,:),6,data.num_atoms);
    data.forceLocal = transpose(forceLocal);
    forceNonLcl = reshape(inForces2(2,:),6,data.num_atoms);
    data.forceNonLcl = transpose(forceNonLcl);
    forceTotal = reshape(inForces2(3,:),6,data.num_atoms);
    data.forceTotal = transpose(forceTotal);
    forceTotal2 = reshape(inForces2(4,:),6,data.num_atoms);
    data.forceTotal2 = transpose(forceTotal2);

    inForces3 = load('pTemp/forces3.dat');
    data.dipole = reshape(inForces3(1,:),1,3);
    data.netForce = reshape(inForces3(2,:),1,3);
    data.netTorque = reshape(inForces3(3,:),1,3);

    D_valid = true;

end

waitbar(0.3,h);

% OPEN WFN.DAT
try
    waitbar(0.4,h,'Opening wfn.dat   ');

    [wfn, pot, rho, wavedata] = wfnread_rsdft(datName);
    
    % wfn.dat read to completion
    W_valid(1) = 1;
    
    waitbar(0.6,h,'Processing wfn.dat');
    p = floor(length(pot)^(1/3)+1);
    
    % Initialize sparse matracies
    wfn.pot = zeros( p,p,p );
    wfn.rho = zeros( p,p,p );
    wfn.wfn = zeros(p,p,p,wfn.num_states);

    W_valid(2) = 1;
        
    % Read pot and rho
    for x=1:p
      for y=1:p
	for z=1:p
	  wfn.pot(y,x,z) = pot(x + p*(y-1) + p*p*(z-1));
	  wfn.rho(y,x,z) = abs(rho(x + p*(y-1) + p*p*(z-1)));
	end
      end
    end

    % pot and rho read correctly
    W_valid(3) = 1;
    waitbar(0.65,h);
    
    % Read wfn's
    for i = (1:wfn.num_states)
      for x=1:p
	for y=1:p
	  for z=1:p
	    wfn.wfn(y,x,z,i) = abs(wavedata(x + p*(y-1) + p*p*(z-1),i));
	  end
	end
      end
    end
    % wfn's read correctly
    W_valid(4) = 1;
    
    wfn.wfn_min = min(min(min(min(wfn.wfn))));
    wfn.wfn_max = max(max(max(max(wfn.wfn))));
    waitbar(0.85,h);

    % Compute number of z slices
    [trash, trash, wfn.n, trash] = size(wfn.wfn);

    q = ( -1*wfn.radius:wfn.step: (wfn.radius));
    [wfn.X,wfn.Y,wfn.Z] = meshgrid(q,q,q);
    
    % meshgrid created correctly
    W_valid(5) = 1;
    
%{    
    wfnid = fopen('./comps.txt', 'w');
    fprintf(wfnid, '%e\n', wfn.pot);
    
    rho_length = length(rho(:,1));
    fprintf(wfnid, '%d\n', rho_length);
    fprintf(wfnid, '%e\n', wfn.rho);
    
    fprintf(wfnid, '%e\n', wfn.wfn);     
%}
end

% Read in Atom Data
try
    waitbar(0.9,h,'Reading Atom Data ');

    % atom data
    data.numTypes = 0;
    i = 1;
    fAtom = fopen('pTemp/atom.dat');
    input = fgetl(fAtom);
    count = 0;
    while input ~= -1
        data.numTypes = data.numTypes + 1;
        inputd = sscanf(mat2str(input), '%d %s');

        % get the name of the atom
        if ( size(inputd) < 3 )
            b = char(inputd(2));
            b(2) = ' ';
        else
            b = char(inputd(2:size(inputd)));
        end

        for ( j = 1:inputd(1) )
            data.atomName(count+j,:) = b;
        end

        data.typeNum(data.numTypes) = inputd(1);
        data.typeName(data.numTypes,:) = b;
        data.typeSize(data.numTypes) = 1;
        switch(b)
            case 'C '
                data.typeColor(data.numTypes,:) = [.3 .3 .3];
            case 'O '
                data.typeColor(data.numTypes,:) = [.938 0 0];
            case 'N '
                data.typeColor(data.numTypes,:) = [.368 .705 1];
            case 'H '
                data.typeColor(data.numTypes,:) = [1 1 1];
            case 'S '
                data.typeColor(data.numTypes,:) = [1 1 0];
            case 'P '
                data.typeColor(data.numTypes,:) = [1 .446 0];
            case 'Cl'
                data.typeColor(data.numTypes,:) = [0 1 0];
            case 'Ca'
                data.typeColor(data.numTypes,:) = [.4 .4 .4];
            otherwise
                data.typeColor(data.numTypes,:) = [.491 0 .385];
        end
	num = inputd(1);
	for i = 1:num
	  input = fgetl(fAtom);
	  inputd = sscanf(mat2str(input), '%f %f %f');
	  data.forceTotal(count+1, 1) = inputd(1);
	  data.forceTotal(count+1, 2) = inputd(2);
	  data.forceTotal(count+1, 3) = inputd(3);
	  count = count + 1;
	end
        input = fgetl(fAtom);
    end
catch

end

fclose('all');

waitbar(1,h);

close(h);
